# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from instana_client.models.software_user import SoftwareUser
from typing import Optional, Set
from typing_extensions import Self

class SoftwareVersion(BaseModel):
    """
    SoftwareVersion
    """ # noqa: E501
    discovery_type: StrictStr = Field(alias="discoveryType")
    metadata: Optional[Dict[str, StrictStr]] = None
    name: StrictStr
    plugin: StrictStr
    software_type: StrictStr = Field(alias="softwareType")
    used_by: List[SoftwareUser] = Field(alias="usedBy")
    vendor: StrictStr
    version: StrictStr
    __properties: ClassVar[List[str]] = ["discoveryType", "metadata", "name", "plugin", "softwareType", "usedBy", "vendor", "version"]

    @field_validator('discovery_type')
    def discovery_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['NATIVE_SENSOR', 'REMOTE_SENSOR', 'PACKAGE_MANAGER', 'OTHER']):
            raise ValueError("must be one of enum values ('NATIVE_SENSOR', 'REMOTE_SENSOR', 'PACKAGE_MANAGER', 'OTHER')")
        return value

    @field_validator('software_type')
    def software_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['DEPENDENCY', 'RUNTIME', 'DATABASE', 'MESSAGING', 'PLATFORM', 'PACKAGE', 'OS', 'CACHE', 'API_GATEWAY', 'OTHER', 'APPLICATION_FRAMEWORK']):
            raise ValueError("must be one of enum values ('DEPENDENCY', 'RUNTIME', 'DATABASE', 'MESSAGING', 'PLATFORM', 'PACKAGE', 'OS', 'CACHE', 'API_GATEWAY', 'OTHER', 'APPLICATION_FRAMEWORK')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SoftwareVersion from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in used_by (list)
        _items = []
        if self.used_by:
            for _item_used_by in self.used_by:
                if _item_used_by:
                    _items.append(_item_used_by.to_dict())
            _dict['usedBy'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SoftwareVersion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "discoveryType": obj.get("discoveryType"),
            "metadata": obj.get("metadata"),
            "name": obj.get("name"),
            "plugin": obj.get("plugin"),
            "softwareType": obj.get("softwareType"),
            "usedBy": [SoftwareUser.from_dict(_item) for _item in obj["usedBy"]] if obj.get("usedBy") is not None else None,
            "vendor": obj.get("vendor"),
            "version": obj.get("version")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.api.action_history_api import ActionHistoryApi


class TestActionHistoryApi(unittest.TestCase):
    """ActionHistoryApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ActionHistoryApi()

    def tearDown(self) -> None:
        pass

    def test_add_action_instance(self) -> None:
        """Test case for add_action_instance

        Run an automation action.
        """
        pass

    def test_delete_action_instance(self) -> None:
        """Test case for delete_action_instance

        Deletes an automation action run result from the action run history by ID.
        """
        pass

    def test_delete_action_instances(self) -> None:
        """Test case for delete_action_instances

        Delete automation action run results.
        """
        pass

    def test_get_action_instance(self) -> None:
        """Test case for get_action_instance

        Get the details of an automation action run result by ID from action run history.
        """
        pass

    def test_get_action_instances(self) -> None:
        """Test case for get_action_instances

        Get the details of automation action run results from action run history.
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.hyper_param import HyperParam
from instana_client.models.rule_input import RuleInput
from typing import Optional, Set
from typing_extensions import Self

class BuiltInEventSpecification(BaseModel):
    """
    BuiltInEventSpecification
    """ # noqa: E501
    description: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=2048)]] = Field(default=None, description="Description of Built-in Event Specification")
    enabled: Optional[StrictBool] = Field(default=None, description="Flag to show whether the Built-in Event Specification is enabled")
    hidden: Optional[StrictBool] = Field(default=None, description="Flag to show whether the Built-in Event Specification is hidden")
    hyper_params: Annotated[List[HyperParam], Field(min_length=0, max_length=32)] = Field(description="List of hyper parameters of the Built-in Event Specification", alias="hyperParams")
    id: Annotated[str, Field(min_length=0, strict=True, max_length=2048)] = Field(description="ID of Built-in Event Specification")
    name: Annotated[str, Field(min_length=0, strict=True, max_length=256)] = Field(description="Name of Built-in Event Specification")
    rule_inputs: Annotated[List[RuleInput], Field(min_length=0, max_length=32)] = Field(description="List of input rules of the Built-in Event Specification", alias="ruleInputs")
    severity: Optional[StrictInt] = Field(default=None, description="Severity level of Built-in Event Specification")
    short_plugin_id: Annotated[str, Field(min_length=0, strict=True, max_length=64)] = Field(description="ID of short plugin of Built-in Event Specification", alias="shortPluginId")
    triggering: Optional[StrictBool] = Field(default=None, description="Flag to show whether the Built-in Event Specification is triggering")
    __properties: ClassVar[List[str]] = ["description", "enabled", "hidden", "hyperParams", "id", "name", "ruleInputs", "severity", "shortPluginId", "triggering"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BuiltInEventSpecification from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in hyper_params (list)
        _items = []
        if self.hyper_params:
            for _item_hyper_params in self.hyper_params:
                if _item_hyper_params:
                    _items.append(_item_hyper_params.to_dict())
            _dict['hyperParams'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in rule_inputs (list)
        _items = []
        if self.rule_inputs:
            for _item_rule_inputs in self.rule_inputs:
                if _item_rule_inputs:
                    _items.append(_item_rule_inputs.to_dict())
            _dict['ruleInputs'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BuiltInEventSpecification from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "description": obj.get("description"),
            "enabled": obj.get("enabled"),
            "hidden": obj.get("hidden"),
            "hyperParams": [HyperParam.from_dict(_item) for _item in obj["hyperParams"]] if obj.get("hyperParams") is not None else None,
            "id": obj.get("id"),
            "name": obj.get("name"),
            "ruleInputs": [RuleInput.from_dict(_item) for _item in obj["ruleInputs"]] if obj.get("ruleInputs") is not None else None,
            "severity": obj.get("severity"),
            "shortPluginId": obj.get("shortPluginId"),
            "triggering": obj.get("triggering")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from importlib import import_module
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from instana_client.models.application_event_result import ApplicationEventResult
    from instana_client.models.endpoint_event_result import EndpointEventResult
    from instana_client.models.infra_event_result import InfraEventResult
    from instana_client.models.log_event_result import LogEventResult
    from instana_client.models.mobile_app_event_result import MobileAppEventResult
    from instana_client.models.service_event_result import ServiceEventResult
    from instana_client.models.synthetics_event_result import SyntheticsEventResult
    from instana_client.models.website_event_result import WebsiteEventResult

class EventResult(BaseModel):
    """
    EventResult
    """ # noqa: E501
    detail: Optional[StrictStr] = Field(default=None, description="Details of the event.")
    end: Optional[StrictInt] = Field(default=None, description="A Unix timestamp representing the end time of the Event.")
    entity_label: Optional[StrictStr] = Field(default=None, description="The label of the affected entity of the event.", alias="entityLabel")
    entity_name: Optional[StrictStr] = Field(default=None, description="The name or type of the affected entity of the event.", alias="entityName")
    entity_type: Optional[StrictStr] = Field(default=None, description="The category of the affected entity.", alias="entityType")
    event_id: Optional[StrictStr] = Field(default=None, description="ID of this Event.", alias="eventId")
    event_specification_id: Optional[StrictStr] = Field(default=None, description="ID of the configuration that created this event.", alias="eventSpecificationId")
    fix_suggestion: Optional[StrictStr] = Field(default=None, description="Details of the event.", alias="fixSuggestion")
    metrics: Optional[List[Dict[str, Dict[str, Any]]]] = Field(default=None, description="List of metrics associated with the Event.")
    probable_cause: Optional[Dict[str, Dict[str, Any]]] = Field(default=None, description="Metadata of the probable root cause for this event. Only present in case of specific \"Incident\" type events.", alias="probableCause")
    problem: Optional[StrictStr] = Field(default=None, description="Main problem title of the Event.")
    recent_events: Optional[List[Dict[str, Dict[str, Any]]]] = Field(default=None, description="List of related recent events. Only present in case of \"Incident\" type events.", alias="recentEvents")
    severity: Optional[StrictInt] = Field(default=None, description="The severity of the Event when triggered.")
    short_code: Optional[StrictStr] = Field(default=None, description="A short code for the event", alias="shortCode")
    snapshot_id: Optional[StrictStr] = Field(default=None, description="The snapshot ID of the affected entity of this event.", alias="snapshotId")
    start: Optional[StrictInt] = Field(default=None, description="A Unix timestamp representing the start time of the Event.")
    state: Optional[StrictStr] = Field(default=None, description="The state of the Event, \"open\" or \"close\". ")
    type: Optional[StrictStr] = Field(default=None, description="The type of Event.")
    __properties: ClassVar[List[str]] = ["detail", "end", "entityLabel", "entityName", "entityType", "eventId", "eventSpecificationId", "fixSuggestion", "metrics", "probableCause", "problem", "recentEvents", "severity", "shortCode", "snapshotId", "start", "state", "type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = 'entityType'

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        'APPLICATION': 'ApplicationEventResult','ENDPOINT': 'EndpointEventResult','INFRASTRUCTURE': 'InfraEventResult','LOG': 'LogEventResult','MOBILE_APP': 'MobileAppEventResult','SERVICE': 'ServiceEventResult','SYNTHETICS': 'SyntheticsEventResult','WEBSITE': 'WebsiteEventResult'
    }

    @classmethod
    def get_discriminator_value(cls, obj: Dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Union[ApplicationEventResult, EndpointEventResult, InfraEventResult, LogEventResult, MobileAppEventResult, ServiceEventResult, SyntheticsEventResult, WebsiteEventResult]]:
        """Create an instance of EventResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Optional[Union[ApplicationEventResult, EndpointEventResult, InfraEventResult, LogEventResult, MobileAppEventResult, ServiceEventResult, SyntheticsEventResult, WebsiteEventResult]]:
        """Create an instance of EventResult from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type ==  'ApplicationEventResult':
            return import_module("instana_client.models.application_event_result").ApplicationEventResult.from_dict(obj)
        if object_type ==  'EndpointEventResult':
            return import_module("instana_client.models.endpoint_event_result").EndpointEventResult.from_dict(obj)
        if object_type ==  'InfraEventResult':
            return import_module("instana_client.models.infra_event_result").InfraEventResult.from_dict(obj)
        if object_type ==  'LogEventResult':
            return import_module("instana_client.models.log_event_result").LogEventResult.from_dict(obj)
        if object_type ==  'MobileAppEventResult':
            return import_module("instana_client.models.mobile_app_event_result").MobileAppEventResult.from_dict(obj)
        if object_type ==  'ServiceEventResult':
            return import_module("instana_client.models.service_event_result").ServiceEventResult.from_dict(obj)
        if object_type ==  'SyntheticsEventResult':
            return import_module("instana_client.models.synthetics_event_result").SyntheticsEventResult.from_dict(obj)
        if object_type ==  'WebsiteEventResult':
            return import_module("instana_client.models.website_event_result").WebsiteEventResult.from_dict(obj)

        raise ValueError("EventResult failed to lookup discriminator value from " +
                            json.dumps(obj) + ". Discriminator property name: " + cls.__discriminator_property_name +
                            ", mapping: " + json.dumps(cls.__discriminator_value_class_map))





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.slo_report import SloReport

class TestSloReport(unittest.TestCase):
    """SloReport unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SloReport:
        """Test SloReport
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SloReport`
        """
        model = SloReport()
        if include_optional:
            return SloReport(
                error_accumulation_chart = {
                    'key' : 56
                    },
                error_budget_remain_chart = {
                    'key' : 56
                    },
                error_budget_remaining = 56,
                error_budget_spent = 56,
                error_burn_rate = 56,
                error_burn_rate_chart = {
                    'key' : 56
                    },
                error_chart = {
                    'key' : 56
                    },
                from_timestamp = 56,
                sli = 1.337,
                slo = 1.337,
                to_timestamp = 56,
                total_error_budget = 56,
                violation_distribution = {
                    'key' : 56
                    }
            )
        else:
            return SloReport(
        )
        """

    def testSloReport(self):
        """Test SloReport"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.api.service_levels_objective_slo_configurations_api import ServiceLevelsObjectiveSLOConfigurationsApi


class TestServiceLevelsObjectiveSLOConfigurationsApi(unittest.TestCase):
    """ServiceLevelsObjectiveSLOConfigurationsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ServiceLevelsObjectiveSLOConfigurationsApi()

    def tearDown(self) -> None:
        pass

    def test_create_slo_config(self) -> None:
        """Test case for create_slo_config

        Create a new SLO Config
        """
        pass

    def test_delete_slo_config(self) -> None:
        """Test case for delete_slo_config

        Delete an existing SLO Config
        """
        pass

    def test_get_all_slo_config_tags(self) -> None:
        """Test case for get_all_slo_config_tags

        Get All SLO Config tags
        """
        pass

    def test_get_all_slo_configs(self) -> None:
        """Test case for get_all_slo_configs

        Get All SLO Configs
        """
        pass

    def test_get_slo_config_by_id(self) -> None:
        """Test case for get_slo_config_by_id

        Get an existing SLO Config
        """
        pass

    def test_update_slo_config(self) -> None:
        """Test case for update_slo_config

        Update an existing SLO Config
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.trace_result import TraceResult

class TestTraceResult(unittest.TestCase):
    """TraceResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> TraceResult:
        """Test TraceResult
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TraceResult`
        """
        model = TraceResult()
        if include_optional:
            return TraceResult(
                adjusted_timeframe = instana_client.models.adjusted_timeframe.AdjustedTimeframe(
                    to = 56, 
                    window_size = 0, ),
                can_load_more = True,
                items = [
                    instana_client.models.trace_item.TraceItem(
                        cursor = instana_client.models.cursor.cursor(), 
                        trace = instana_client.models.trace.Trace(
                            duration = 0, 
                            endpoint = instana_client.models.endpoint.Endpoint(
                                entity_type = 'APPLICATION', 
                                id = '', 
                                is_synthetic = True, 
                                label = '', 
                                service_id = '', 
                                synthetic = True, 
                                synthetic_type = 'NON_SYNTHETIC', 
                                technologies = [
                                    ''
                                    ], 
                                type = 'UNDEFINED', ), 
                            erroneous = True, 
                            id = '', 
                            label = '', 
                            service = instana_client.models.service.Service(
                                entity_type = 'APPLICATION', 
                                id = '', 
                                label = '', 
                                snapshot_ids = [
                                    ''
                                    ], 
                                technologies = [
                                    ''
                                    ], 
                                types = [
                                    'UNDEFINED'
                                    ], ), 
                            start_time = 1, ), )
                    ],
                total_hits = 0,
                total_represented_item_count = 0,
                total_retained_item_count = 0
            )
        else:
            return TraceResult(
                items = [
                    instana_client.models.trace_item.TraceItem(
                        cursor = instana_client.models.cursor.cursor(), 
                        trace = instana_client.models.trace.Trace(
                            duration = 0, 
                            endpoint = instana_client.models.endpoint.Endpoint(
                                entity_type = 'APPLICATION', 
                                id = '', 
                                is_synthetic = True, 
                                label = '', 
                                service_id = '', 
                                synthetic = True, 
                                synthetic_type = 'NON_SYNTHETIC', 
                                technologies = [
                                    ''
                                    ], 
                                type = 'UNDEFINED', ), 
                            erroneous = True, 
                            id = '', 
                            label = '', 
                            service = instana_client.models.service.Service(
                                entity_type = 'APPLICATION', 
                                id = '', 
                                label = '', 
                                snapshot_ids = [
                                    ''
                                    ], 
                                technologies = [
                                    ''
                                    ], 
                                types = [
                                    'UNDEFINED'
                                    ], ), 
                            start_time = 1, ), )
                    ],
        )
        """

    def testTraceResult(self):
        """Test TraceResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.application_alert_config import ApplicationAlertConfig

class TestApplicationAlertConfig(unittest.TestCase):
    """ApplicationAlertConfig unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ApplicationAlertConfig:
        """Test ApplicationAlertConfig
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ApplicationAlertConfig`
        """
        model = ApplicationAlertConfig()
        if include_optional:
            return ApplicationAlertConfig(
                alert_channel_ids = [
                    ''
                    ],
                alert_channels = {
                    'key' : [
                        ''
                        ]
                    },
                application_id = '',
                applications = {
                    'key' : instana_client.models.application_node.ApplicationNode(
                        application_id = '', 
                        inclusive = True, 
                        services = {
                            'key' : instana_client.models.service_node.ServiceNode(
                                endpoints = {
                                    'key' : instana_client.models.endpoint_node.EndpointNode(
                                        endpoint_id = '', 
                                        inclusive = True, )
                                    }, 
                                inclusive = True, 
                                service_id = '', )
                            }, )
                    },
                boundary_scope = 'ALL',
                custom_payload_fields = [
                    instana_client.models.custom_payload_field.CustomPayloadField(
                        key = '', 
                        type = '', )
                    ],
                description = '',
                evaluation_type = 'PER_AP',
                grace_period = 56,
                granularity = 60000,
                include_internal = True,
                include_synthetic = True,
                name = '',
                rule = instana_client.models.application_alert_rule.ApplicationAlertRule(
                    aggregation = 'SUM', 
                    alert_type = '', 
                    metric_name = '', ),
                rules = [
                    instana_client.models.rule_with_threshold_application_alert_rule.RuleWithThresholdApplicationAlertRule(
                        rule = instana_client.models.application_alert_rule.ApplicationAlertRule(
                            aggregation = 'SUM', 
                            alert_type = '', 
                            metric_name = '', ), 
                        threshold_operator = '>', 
                        thresholds = {
                            'key' : instana_client.models.threshold_config_rule.ThresholdConfigRule(
                                type = '', )
                            }, )
                    ],
                severity = 5,
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                tag_filters = [
                    instana_client.models.tag_filter.TagFilter()
                    ],
                threshold = instana_client.models.threshold.Threshold(
                    operator = '>', 
                    type = '', ),
                time_threshold = instana_client.models.application_time_threshold.ApplicationTimeThreshold(
                    time_window = 56, 
                    type = '', ),
                triggering = True
            )
        else:
            return ApplicationAlertConfig(
                alert_channel_ids = [
                    ''
                    ],
                applications = {
                    'key' : instana_client.models.application_node.ApplicationNode(
                        application_id = '', 
                        inclusive = True, 
                        services = {
                            'key' : instana_client.models.service_node.ServiceNode(
                                endpoints = {
                                    'key' : instana_client.models.endpoint_node.EndpointNode(
                                        endpoint_id = '', 
                                        inclusive = True, )
                                    }, 
                                inclusive = True, 
                                service_id = '', )
                            }, )
                    },
                boundary_scope = 'ALL',
                custom_payload_fields = [
                    instana_client.models.custom_payload_field.CustomPayloadField(
                        key = '', 
                        type = '', )
                    ],
                description = '',
                evaluation_type = 'PER_AP',
                granularity = 60000,
                name = '',
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                time_threshold = instana_client.models.application_time_threshold.ApplicationTimeThreshold(
                    time_window = 56, 
                    type = '', ),
        )
        """

    def testApplicationAlertConfig(self):
        """Test ApplicationAlertConfig"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.endpoint import Endpoint
from instana_client.models.service import Service
from typing import Optional, Set
from typing_extensions import Self

class Trace(BaseModel):
    """
    1. `service`: The service from where trace started. 2. `endpoint`: Endpoint of the service. 
    """ # noqa: E501
    duration: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="Total time taken for a trace to finish.")
    endpoint: Optional[Endpoint] = None
    erroneous: Optional[StrictBool] = Field(default=None, description="Flag which tells whether the trace is erroneous or not")
    id: StrictStr = Field(description="The trace ID. All spans of the same trace must have the same trace ID. For example, `e93282c0d5018320`.")
    label: StrictStr = Field(description="Name of the trace.")
    service: Optional[Service] = None
    start_time: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(default=None, description="The start time of the trace.", alias="startTime")
    __properties: ClassVar[List[str]] = ["duration", "endpoint", "erroneous", "id", "label", "service", "startTime"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Trace from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of endpoint
        if self.endpoint:
            _dict['endpoint'] = self.endpoint.to_dict()
        # override the default output from pydantic by calling `to_dict()` of service
        if self.service:
            _dict['service'] = self.service.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Trace from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "duration": obj.get("duration"),
            "endpoint": Endpoint.from_dict(obj["endpoint"]) if obj.get("endpoint") is not None else None,
            "erroneous": obj.get("erroneous"),
            "id": obj.get("id"),
            "label": obj.get("label"),
            "service": Service.from_dict(obj["service"]) if obj.get("service") is not None else None,
            "startTime": obj.get("startTime")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool
from typing import Optional
from typing_extensions import Annotated
from instana_client.models.application_metric_result import ApplicationMetricResult
from instana_client.models.endpoint_metric_result import EndpointMetricResult
from instana_client.models.get_application_metrics import GetApplicationMetrics
from instana_client.models.get_applications import GetApplications
from instana_client.models.get_endpoints import GetEndpoints
from instana_client.models.get_services import GetServices
from instana_client.models.metric_api_result import MetricAPIResult
from instana_client.models.service_metric_result import ServiceMetricResult

from instana_client.api_client import ApiClient, RequestSerialized
from instana_client.api_response import ApiResponse
from instana_client.rest import RESTResponseType


class ApplicationMetricsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_application_data_metrics_v2(
        self,
        get_application_metrics: Optional[GetApplicationMetrics] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> MetricAPIResult:
        """Get Application Data Metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for a combination of entities. For eg: retrieve `MEAN` aggregation of `latency` metric for an Endpoint `GET /api/foo`, Service `payment` and Application Perspective `app`. Consider this API endpoint an upgraded version of `Get Application Metrics`, `Get Endpoint metrics` and `Get Service metrics`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param get_application_metrics:
        :type get_application_metrics: GetApplicationMetrics
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_data_metrics_v2_serialize(
            get_application_metrics=get_application_metrics,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MetricAPIResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_application_data_metrics_v2_with_http_info(
        self,
        get_application_metrics: Optional[GetApplicationMetrics] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[MetricAPIResult]:
        """Get Application Data Metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for a combination of entities. For eg: retrieve `MEAN` aggregation of `latency` metric for an Endpoint `GET /api/foo`, Service `payment` and Application Perspective `app`. Consider this API endpoint an upgraded version of `Get Application Metrics`, `Get Endpoint metrics` and `Get Service metrics`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param get_application_metrics:
        :type get_application_metrics: GetApplicationMetrics
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_data_metrics_v2_serialize(
            get_application_metrics=get_application_metrics,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MetricAPIResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_application_data_metrics_v2_without_preload_content(
        self,
        get_application_metrics: Optional[GetApplicationMetrics] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Application Data Metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for a combination of entities. For eg: retrieve `MEAN` aggregation of `latency` metric for an Endpoint `GET /api/foo`, Service `payment` and Application Perspective `app`. Consider this API endpoint an upgraded version of `Get Application Metrics`, `Get Endpoint metrics` and `Get Service metrics`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param get_application_metrics:
        :type get_application_metrics: GetApplicationMetrics
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_data_metrics_v2_serialize(
            get_application_metrics=get_application_metrics,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MetricAPIResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_application_data_metrics_v2_serialize(
        self,
        get_application_metrics,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_application_metrics is not None:
            _body_params = get_application_metrics


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/v2/metrics',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_application_metrics(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_applications: Optional[GetApplications] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApplicationMetricResult:
        """Get Application Metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for an Application Perspective. For eg: retrieve `MEAN` aggregation of `latency` metric for an Application Perspective `app`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_applications:
        :type get_applications: GetApplications
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_metrics_serialize(
            fill_time_series=fill_time_series,
            get_applications=get_applications,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ApplicationMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_application_metrics_with_http_info(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_applications: Optional[GetApplications] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ApplicationMetricResult]:
        """Get Application Metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for an Application Perspective. For eg: retrieve `MEAN` aggregation of `latency` metric for an Application Perspective `app`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_applications:
        :type get_applications: GetApplications
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_metrics_serialize(
            fill_time_series=fill_time_series,
            get_applications=get_applications,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ApplicationMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_application_metrics_without_preload_content(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_applications: Optional[GetApplications] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Application Metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for an Application Perspective. For eg: retrieve `MEAN` aggregation of `latency` metric for an Application Perspective `app`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_applications:
        :type get_applications: GetApplications
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_metrics_serialize(
            fill_time_series=fill_time_series,
            get_applications=get_applications,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ApplicationMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_application_metrics_serialize(
        self,
        fill_time_series,
        get_applications,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if fill_time_series is not None:
            
            _query_params.append(('fillTimeSeries', fill_time_series))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_applications is not None:
            _body_params = get_applications


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/metrics/applications',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_endpoints_metrics(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_endpoints: Optional[GetEndpoints] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EndpointMetricResult:
        """Get Endpoint metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for an Endpoint. For eg: retrieve `MEAN` aggregation of `latency` metric for an Endpoint `GET /api/foo`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_endpoints:
        :type get_endpoints: GetEndpoints
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_endpoints_metrics_serialize(
            fill_time_series=fill_time_series,
            get_endpoints=get_endpoints,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EndpointMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_endpoints_metrics_with_http_info(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_endpoints: Optional[GetEndpoints] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EndpointMetricResult]:
        """Get Endpoint metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for an Endpoint. For eg: retrieve `MEAN` aggregation of `latency` metric for an Endpoint `GET /api/foo`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_endpoints:
        :type get_endpoints: GetEndpoints
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_endpoints_metrics_serialize(
            fill_time_series=fill_time_series,
            get_endpoints=get_endpoints,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EndpointMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_endpoints_metrics_without_preload_content(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        get_endpoints: Optional[GetEndpoints] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Endpoint metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for an Endpoint. For eg: retrieve `MEAN` aggregation of `latency` metric for an Endpoint `GET /api/foo`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param get_endpoints:
        :type get_endpoints: GetEndpoints
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_endpoints_metrics_serialize(
            fill_time_series=fill_time_series,
            get_endpoints=get_endpoints,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EndpointMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_endpoints_metrics_serialize(
        self,
        fill_time_series,
        get_endpoints,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if fill_time_series is not None:
            
            _query_params.append(('fillTimeSeries', fill_time_series))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_endpoints is not None:
            _body_params = get_endpoints


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/metrics/endpoints',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_services_metrics(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        include_snapshot_ids: Optional[StrictBool] = None,
        get_services: Optional[GetServices] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ServiceMetricResult:
        """Get Service metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for a Service. For eg: retrieve `MEAN` aggregation of `latency` metric for a Service `payment`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param include_snapshot_ids:
        :type include_snapshot_ids: bool
        :param get_services:
        :type get_services: GetServices
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_services_metrics_serialize(
            fill_time_series=fill_time_series,
            include_snapshot_ids=include_snapshot_ids,
            get_services=get_services,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ServiceMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_services_metrics_with_http_info(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        include_snapshot_ids: Optional[StrictBool] = None,
        get_services: Optional[GetServices] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ServiceMetricResult]:
        """Get Service metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for a Service. For eg: retrieve `MEAN` aggregation of `latency` metric for a Service `payment`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param include_snapshot_ids:
        :type include_snapshot_ids: bool
        :param get_services:
        :type get_services: GetServices
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_services_metrics_serialize(
            fill_time_series=fill_time_series,
            include_snapshot_ids=include_snapshot_ids,
            get_services=get_services,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ServiceMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_services_metrics_without_preload_content(
        self,
        fill_time_series: Annotated[Optional[StrictBool], Field(description="If enabled, fill the missing data points in the metric result with timestamp and value 0.")] = None,
        include_snapshot_ids: Optional[StrictBool] = None,
        get_services: Optional[GetServices] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Service metrics

        Use this API endpoint if one wants to retrieve one or more supported aggregation of supported metrics for a Service. For eg: retrieve `MEAN` aggregation of `latency` metric for a Service `payment`. For more information on supported metrics, refer `Get Metric catalog`.  For more information on Application Metrics please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-metrics.

        :param fill_time_series: If enabled, fill the missing data points in the metric result with timestamp and value 0.
        :type fill_time_series: bool
        :param include_snapshot_ids:
        :type include_snapshot_ids: bool
        :param get_services:
        :type get_services: GetServices
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_services_metrics_serialize(
            fill_time_series=fill_time_series,
            include_snapshot_ids=include_snapshot_ids,
            get_services=get_services,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ServiceMetricResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_services_metrics_serialize(
        self,
        fill_time_series,
        include_snapshot_ids,
        get_services,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if fill_time_series is not None:
            
            _query_params.append(('fillTimeSeries', fill_time_series))
            
        if include_snapshot_ids is not None:
            
            _query_params.append(('includeSnapshotIds', include_snapshot_ids))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_services is not None:
            _body_params = get_services


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/application-monitoring/metrics/services',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.304.1059
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.service_level_objective_configuration import ServiceLevelObjectiveConfiguration

class TestServiceLevelObjectiveConfiguration(unittest.TestCase):
    """ServiceLevelObjectiveConfiguration unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ServiceLevelObjectiveConfiguration:
        """Test ServiceLevelObjectiveConfiguration
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ServiceLevelObjectiveConfiguration`
        """
        model = ServiceLevelObjectiveConfiguration()
        if include_optional:
            return ServiceLevelObjectiveConfiguration(
                created_date = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                entity = instana_client.models.slo_entity.SloEntity(
                    tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                        type = '', ), 
                    type = '', ),
                id = '',
                indicator = instana_client.models.service_level_indicator.ServiceLevelIndicator(
                    aggregation = 'SUM', 
                    operator = '>', 
                    service_levels_measurement = 'eventBased', 
                    threshold = 1.337, 
                    type = 'eventBased', ),
                last_updated = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                name = '',
                tags = [
                    ''
                    ],
                target = 0.0,
                time_window = instana_client.models.time_window.TimeWindow(
                    duration = 56, 
                    duration_unit = 'millisecond', 
                    timezone = '', 
                    type = '', )
            )
        else:
            return ServiceLevelObjectiveConfiguration(
                entity = instana_client.models.slo_entity.SloEntity(
                    tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                        type = '', ), 
                    type = '', ),
                indicator = instana_client.models.service_level_indicator.ServiceLevelIndicator(
                    aggregation = 'SUM', 
                    operator = '>', 
                    service_levels_measurement = 'eventBased', 
                    threshold = 1.337, 
                    type = 'eventBased', ),
                name = '',
                tags = [
                    ''
                    ],
                target = 0.0,
                time_window = instana_client.models.time_window.TimeWindow(
                    duration = 56, 
                    duration_unit = 'millisecond', 
                    timezone = '', 
                    type = '', ),
        )
        """

    def testServiceLevelObjectiveConfiguration(self):
        """Test ServiceLevelObjectiveConfiguration"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.built_in_event_specification import BuiltInEventSpecification

class TestBuiltInEventSpecification(unittest.TestCase):
    """BuiltInEventSpecification unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> BuiltInEventSpecification:
        """Test BuiltInEventSpecification
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `BuiltInEventSpecification`
        """
        model = BuiltInEventSpecification()
        if include_optional:
            return BuiltInEventSpecification(
                description = '',
                enabled = True,
                hidden = True,
                hyper_params = [
                    instana_client.models.hyper_param.HyperParam(
                        default_value = 1.337, 
                        description = '', 
                        id = '', 
                        max_value = 1.337, 
                        min_value = 1.337, 
                        name = '', 
                        value_format = 'NUMBER', )
                    ],
                id = '',
                name = '',
                rule_inputs = [
                    instana_client.models.rule_input.RuleInput(
                        input_kind = 'METRIC', 
                        input_name = '', )
                    ],
                severity = 56,
                short_plugin_id = '',
                triggering = True
            )
        else:
            return BuiltInEventSpecification(
                hyper_params = [
                    instana_client.models.hyper_param.HyperParam(
                        default_value = 1.337, 
                        description = '', 
                        id = '', 
                        max_value = 1.337, 
                        min_value = 1.337, 
                        name = '', 
                        value_format = 'NUMBER', )
                    ],
                id = '',
                name = '',
                rule_inputs = [
                    instana_client.models.rule_input.RuleInput(
                        input_kind = 'METRIC', 
                        input_name = '', )
                    ],
                short_plugin_id = '',
        )
        """

    def testBuiltInEventSpecification(self):
        """Test BuiltInEventSpecification"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import ConfigDict
from typing import Any, ClassVar, Dict, List, Optional
from instana_client.models.synthetic_resource_update_list_string_list_string import SyntheticResourceUpdateListStringListString
from instana_client.models.synthetic_resource_update_map_string_string_list_string import SyntheticResourceUpdateMapStringStringListString
from instana_client.models.synthetic_test_update import SyntheticTestUpdate
from instana_client.models.synthetic_type_configuration_update import SyntheticTypeConfigurationUpdate
from typing import Optional, Set
from typing_extensions import Self

class SyntheticTestDeepUpdate(SyntheticTestUpdate):
    """
    SyntheticTestDeepUpdate
    """ # noqa: E501
    configuration: Optional[SyntheticTypeConfigurationUpdate] = None
    __properties: ClassVar[List[str]] = ["active", "applications", "customProperties", "ids", "lastModifiedAt", "locations", "mobileApps", "modifiedBy", "shallowUpdate", "syntheticUpdateType", "testFrequency", "websites", "configuration"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SyntheticTestDeepUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of applications
        if self.applications:
            _dict['applications'] = self.applications.to_dict()
        # override the default output from pydantic by calling `to_dict()` of custom_properties
        if self.custom_properties:
            _dict['customProperties'] = self.custom_properties.to_dict()
        # override the default output from pydantic by calling `to_dict()` of locations
        if self.locations:
            _dict['locations'] = self.locations.to_dict()
        # override the default output from pydantic by calling `to_dict()` of mobile_apps
        if self.mobile_apps:
            _dict['mobileApps'] = self.mobile_apps.to_dict()
        # override the default output from pydantic by calling `to_dict()` of websites
        if self.websites:
            _dict['websites'] = self.websites.to_dict()
        # override the default output from pydantic by calling `to_dict()` of configuration
        if self.configuration:
            _dict['configuration'] = self.configuration.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SyntheticTestDeepUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "active": obj.get("active"),
            "applications": SyntheticResourceUpdateListStringListString.from_dict(obj["applications"]) if obj.get("applications") is not None else None,
            "customProperties": SyntheticResourceUpdateMapStringStringListString.from_dict(obj["customProperties"]) if obj.get("customProperties") is not None else None,
            "ids": obj.get("ids"),
            "lastModifiedAt": obj.get("lastModifiedAt"),
            "locations": SyntheticResourceUpdateListStringListString.from_dict(obj["locations"]) if obj.get("locations") is not None else None,
            "mobileApps": SyntheticResourceUpdateListStringListString.from_dict(obj["mobileApps"]) if obj.get("mobileApps") is not None else None,
            "modifiedBy": obj.get("modifiedBy"),
            "shallowUpdate": obj.get("shallowUpdate"),
            "syntheticUpdateType": obj.get("syntheticUpdateType"),
            "testFrequency": obj.get("testFrequency"),
            "websites": SyntheticResourceUpdateListStringListString.from_dict(obj["websites"]) if obj.get("websites") is not None else None,
            "configuration": SyntheticTypeConfigurationUpdate.from_dict(obj["configuration"]) if obj.get("configuration") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.get_test_summary_result import GetTestSummaryResult

class TestGetTestSummaryResult(unittest.TestCase):
    """GetTestSummaryResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> GetTestSummaryResult:
        """Test GetTestSummaryResult
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `GetTestSummaryResult`
        """
        model = GetTestSummaryResult()
        if include_optional:
            return GetTestSummaryResult(
                metrics = [
                    instana_client.models.synthetic_metric_configuration.SyntheticMetricConfiguration(
                        aggregation = 'SUM', 
                        granularity = 56, 
                        metric = '', )
                    ],
                order = instana_client.models.order.Order(
                    by = '', 
                    collation = '', 
                    direction = 'ASC', ),
                pagination = instana_client.models.pagination.Pagination(
                    page = 1, 
                    page_size = 1, ),
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                tag_filters = [
                    instana_client.models.tag_filter.TagFilter()
                    ],
                time_frame = instana_client.models.time_frame.TimeFrame(
                    to = 56, 
                    window_size = 0, )
            )
        else:
            return GetTestSummaryResult(
                metrics = [
                    instana_client.models.synthetic_metric_configuration.SyntheticMetricConfiguration(
                        aggregation = 'SUM', 
                        granularity = 56, 
                        metric = '', )
                    ],
        )
        """

    def testGetTestSummaryResult(self):
        """Test GetTestSummaryResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.api.custom_entities_api import CustomEntitiesApi


class TestCustomEntitiesApi(unittest.TestCase):
    """CustomEntitiesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = CustomEntitiesApi()

    def tearDown(self) -> None:
        pass

    def test_create_custom_entities(self) -> None:
        """Test case for create_custom_entities

        Create a Custom Entity type
        """
        pass

    def test_delete_custom_entity(self) -> None:
        """Test case for delete_custom_entity

        Delete a Custom Entity Type
        """
        pass

    def test_get_custom_entity(self) -> None:
        """Test case for get_custom_entity

        Get custom Entity Types
        """
        pass

    def test_list_custom_entities(self) -> None:
        """Test case for list_custom_entities

        List custom Entity Type Definition
        """
        pass

    def test_list_entities_from_infra_entities(self) -> None:
        """Test case for list_entities_from_infra_entities

        List all entity types that match the custom entity metadata
        """
        pass

    def test_update_custom_entity(self) -> None:
        """Test case for update_custom_entity

        Update a Custom Entity Type
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class Tag(BaseModel):
    """
    List of queryable tags available in a tagTree. Eg: `call.erroneous`. Consider these tags as attributes of a tagTree. Eg: `Call` tagTree has have `Erroneous`, `Call name`, `Latency` etc as attributes. 
    """ # noqa: E501
    aliases: Optional[List[StrictStr]] = Field(default=None, description="List of other names that can refer to this tag ")
    availability: Optional[List[StrictStr]] = Field(default=None, description="List of product areas this tag is available in ")
    can_apply_to_destination: Optional[StrictBool] = Field(default=None, description="Whether the tag is available for destination or not. If source and destination is false, it means the tag is independent of source and destination. Eg: of such tag is `call.http.path`. ", alias="canApplyToDestination")
    can_apply_to_source: Optional[StrictBool] = Field(default=None, description="Whether the tag is available for source or not. If source and destination is false, it means the tag is independent of source and destination. Eg: of such tag is `call.http.path`. ", alias="canApplyToSource")
    description: Optional[StrictStr] = Field(default=None, description="The description of the tag if it is provided.")
    id_tag: Optional[StrictBool] = Field(default=None, description="Whether the Tag is a unique ID or not. Eg: `idTag` for `endpoint.id` is true but for `call.rpc.method` it is false. ", alias="idTag")
    label: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=256)]] = Field(default=None, description="The name of the tag which is seen in the UI. Eg: `Call name`")
    name: StrictStr = Field(description="The name of the tag. Eg: `call.name`")
    type: StrictStr = Field(description="The data type of the tag. Eg: `call.name` accepts `STRING` value.")
    __properties: ClassVar[List[str]] = ["aliases", "availability", "canApplyToDestination", "canApplyToSource", "description", "idTag", "label", "name", "type"]

    @field_validator('availability')
    def availability_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['INFRASTRUCTURE_METRICS', 'APPLICATION', 'WEBSITE', 'MOBILE_APP', 'EUM', 'EUM_IMPACTED_BEACON', 'EVENT', 'SLI', 'SLO', 'SLO_PREVIEW', 'USAGE', 'LOG', 'SYNTHETICS', 'SYNTHETICS_DETAIL', 'APDEX', 'BIZOPS', 'BUSINESS_METRICS', 'SUBTRACE', 'UNKNOWN']):
                raise ValueError("each list item must be one of ('INFRASTRUCTURE_METRICS', 'APPLICATION', 'WEBSITE', 'MOBILE_APP', 'EUM', 'EUM_IMPACTED_BEACON', 'EVENT', 'SLI', 'SLO', 'SLO_PREVIEW', 'USAGE', 'LOG', 'SYNTHETICS', 'SYNTHETICS_DETAIL', 'APDEX', 'BIZOPS', 'BUSINESS_METRICS', 'SUBTRACE', 'UNKNOWN')")
        return value

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['BOOLEAN', 'STRING', 'NUMBER', 'STRING_SET', 'STRING_LIST', 'KEY_VALUE_PAIR', 'FLOAT_LIST', 'KEY_NUMBER_PAIR']):
            raise ValueError("must be one of enum values ('BOOLEAN', 'STRING', 'NUMBER', 'STRING_SET', 'STRING_LIST', 'KEY_VALUE_PAIR', 'FLOAT_LIST', 'KEY_NUMBER_PAIR')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Tag from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Tag from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "aliases": obj.get("aliases"),
            "availability": obj.get("availability"),
            "canApplyToDestination": obj.get("canApplyToDestination"),
            "canApplyToSource": obj.get("canApplyToSource"),
            "description": obj.get("description"),
            "idTag": obj.get("idTag"),
            "label": obj.get("label"),
            "name": obj.get("name"),
            "type": obj.get("type")
        })
        return _obj


